@@ -54,7 +54,6 @@ void completion(const char *, const char *, const char *);
 void idutils(const char *, const char *);
 void grep(const char *, const char *);
 void pathlist(const char *, const char *);
-void parsefile(int, char *const *, const char *, const char *, const char *, int);
 void parsefile_using_builtin_parser(char *const *, const char *, const char *, const char *, int);
 int search(const char *, const char *, const char *, const char *, int);
 void tagsearch(const char *, const char *, const char *, const char *, int);
@@ -89,9 +88,9 @@ int nosource;				/* undocumented command */
 int debug;
 int format;
 int type;				/* path conversion type */
-char cwd[MAXPATHLEN+1];			/* current directory	*/
-char root[MAXPATHLEN+1];		/* root of source tree	*/
-char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
+char cwd[MAXPATHLEN];			/* current directory	*/
+char root[MAXPATHLEN];			/* root of source tree	*/
+char dbpath[MAXPATHLEN];		/* dbpath directory	*/
 char *context_file;
 char *context_lineno;
 
@@ -112,6 +111,7 @@ help(void)
 
 #define RESULT		128
 #define FROM_HERE	129
+#define ENCODE_PATH	130
 #define SORT_FILTER     1
 #define PATH_FILTER     2
 #define BOTH_FILTER     (SORT_FILTER|PATH_FILTER)
@@ -142,6 +142,7 @@ static struct option const long_options[] = {
 	{"cxref", no_argument, NULL, 'x'},
 
 	/* long name only */
+	{"encode-path", required_argument, NULL, ENCODE_PATH},
 	{"from-here", required_argument, NULL, FROM_HERE},
 	{"debug", no_argument, &debug, 1},
 	{"version", no_argument, &show_version, 1},
@@ -175,7 +176,7 @@ setcom(int c)
 int
 decide_tag_by_context(const char *tag, const char *file, int lineno)
 {
-	char path[MAXPATHLEN+1], s_fid[32];
+	char path[MAXPATHLEN], s_fid[MAXFIDLEN];
 	const char *tagline, *p;
 	DBOP *dbop;
 	int db = GSYMS;
@@ -353,6 +354,13 @@ main(int argc, char **argv)
 		case 'x':
 			xflag++;
 			break;
+		case ENCODE_PATH:
+			if (strlen(optarg) > 255)
+				die("too many encode chars.");
+			if (strchr(optarg, '/') || strchr(optarg, '.'))
+				die("cannot encode '/' and '.' in the path.");
+			set_encode_chars((unsigned char *)optarg);
+			break;
 		case FROM_HERE:
 			{
 			char *p = optarg;
@@ -406,11 +414,6 @@ main(int argc, char **argv)
 
 	if (show_version)
 		version(av, vflag);
-	/*
-	 * invalid options.
-	 */
-	if (sflag && rflag)
-		die_with_code(2, "both of -s and -r are not allowed.");
 	/*
 	 * only -c, -u, -P and -p allows no argument.
 	 */
@@ -528,7 +531,10 @@ main(int argc, char **argv)
 			die_with_code(2, "regular expression is not allowed with the --from-here option.");
 		db = decide_tag_by_context(av, context_file, atoi(context_lineno));
 	} else {
-		db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
+		if (rflag && sflag)
+			db = GRTAGS + GSYMS;
+		else
+			db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
 	}
 	/*
 	 * decide format.
@@ -577,13 +583,7 @@ main(int argc, char **argv)
 	 * parse source files.
 	 */
 	else if (fflag) {
-		/*
-		 * parserfile() is obsolete.
-		 */
-		if (getconfs("GTAGS", NULL))
-			parsefile(argc, argv, cwd, root, dbpath, db);
-		else
-			parsefile_using_builtin_parser(argv, cwd, root, dbpath, db);
+		parsefile_using_builtin_parser(argv, cwd, root, dbpath, db);
 	}
 	/*
 	 * tag search.
@@ -724,7 +724,7 @@ idutils(const char *pattern, const char *dbpath)
 	FILE *ip;
 	CONVERT *cv;
 	STRBUF *ib = strbuf_open(0);
-	char edit[IDENTLEN+1];
+	char edit[IDENTLEN];
 	const char *path, *lno, *lid;
 	int linenum, count;
 	char *p, *grep;
@@ -742,7 +742,7 @@ idutils(const char *pattern, const char *dbpath)
 	 */
 	strbuf_puts(ib, lid);
 	strbuf_puts(ib, " --separator=newline");
-	if (!tflag && !xflag)
+	if (format == FORMAT_PATH)
 		strbuf_puts(ib, " --result=filenames --key=none");
 	else
 		strbuf_puts(ib, " --result=grep");
@@ -774,7 +774,7 @@ idutils(const char *pattern, const char *dbpath)
 		count++;
 		switch (format) {
 		case FORMAT_PATH:
-			convert_put(cv, path);
+			convert_put_path(cv, path);
 			break;
 		default:
 			/* extract line number */
@@ -792,7 +792,7 @@ idutils(const char *pattern, const char *dbpath)
 			/*
 			 * print out.
 			 */
-			convert_put_using(cv, edit, path, linenum, p);
+			convert_put_using(cv, edit, path, linenum, p, NULL);
 			break;
 		}
 	}
@@ -818,7 +818,7 @@ grep(const char *pattern, const char *dbpath)
 	GFIND *gp;
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
 	const char *path;
-	char edit[IDENTLEN+1];
+	char edit[IDENTLEN];
 	const char *buffer;
 	int linenum, count;
 	int flags = 0;
@@ -853,10 +853,10 @@ grep(const char *pattern, const char *dbpath)
 			if (regexec(&preg, buffer, 0, 0, 0) == 0) {
 				count++;
 				if (format == FORMAT_PATH) {
-					convert_put(cv, path);
+					convert_put_path(cv, path);
 					break;
 				} else {
-					convert_put_using(cv, edit, path, linenum, buffer);
+					convert_put_using(cv, edit, path, linenum, buffer, gp->dbop->lastdat);
 				}
 			}
 		}
@@ -892,7 +892,7 @@ pathlist(const char *pattern, const char *dbpath)
 		target = GPATH_OTHER;
 	if (pattern) {
 		int flags = 0;
-		char edit[IDENTLEN+1];
+		char edit[IDENTLEN];
 
 		if (!Gflag)
 			flags |= REG_EXTENDED;
@@ -925,9 +925,9 @@ pathlist(const char *pattern, const char *dbpath)
 		if (pattern && regexec(&preg, p, 0, 0, 0) != 0)
 			continue;
 		if (format == FORMAT_PATH)
-			convert_put(cv, path);
+			convert_put_path(cv, path);
 		else
-			convert_put_using(cv, "path", path, 1, " ");
+			convert_put_using(cv, "path", path, 1, " ", gp->dbop->lastdat);
 		count++;
 	}
 	gfind_close(gp);
@@ -949,132 +949,6 @@ pathlist(const char *pattern, const char *dbpath)
 		fprintf(stderr, " (using '%s').\n", makepath(dbpath, dbname(GPATH), NULL));
 	}
 }
-/*
- * parsefile: parse file to pick up tags.
- *
- *	i)	argc
- *	i)	argv
- *	i)	cwd	current directory
- *	i)	root	root directory of source tree
- *	i)	dbpath	dbpath
- *	i)	db	type of parse
- */
-void
-parsefile(int argc, char *const *argv, const char *cwd, const char *root, const char *dbpath, int db)
-{
-	CONVERT *cv;
-	int count = 0;
-	int file_count = 0;
-	STRBUF *comline = strbuf_open(0);
-	STRBUF *path_list = strbuf_open(MAXPATHLEN);
-	XARGS *xp;
-	char *ctags_x;
-
-	/*
-	 * teach parser where is dbpath.
-	 */
-	set_env("GTAGSDBPATH", dbpath);
-	/*
-	 * teach parser language mapping.
-	 */
-	{
-		STRBUF *sb = strbuf_open(0);
-
-		if (getconfs("langmap", sb))
-			set_env("GTAGSLANGMAP", strbuf_value(sb));
-		strbuf_close(sb);
-	}
-	/*
-	 * get parser.
-	 */
-	if (!getconfs(dbname(db), comline))
-		die("cannot get parser for %s.", dbname(db));
-	cv = convert_open(type, format, root, cwd, dbpath, stdout);
-	if (gpath_open(dbpath, 0) < 0)
-		die("GPATH not found.");
-	/*
-	 * Make a path list while checking the validity of path name.
-	 */
-	for (; argc > 0; argv++, argc--) {
-		const char *av = argv[0];
-		char path[MAXPATHLEN+1];
-
-		/*
-		 * convert the path into relative to the root directory of source tree.
-		 */
-		if (normalize(av, get_root_with_slash(), cwd, path, sizeof(path)) == NULL)
-			if (!qflag)
-				fprintf(stderr, "'%s' is out of source tree.\n", path + 2);
-		if (!gpath_path2fid(path, NULL)) {
-			if (!qflag)
-				fprintf(stderr, "'%s' not found in GPATH.\n", path + 2);
-			continue;
-		}
-		if (!test("f", makepath(root, path, NULL))) {
-			if (test("d", NULL)) {
-				if (!qflag)
-					fprintf(stderr, "'%s' is a directory.\n", av);
-			} else {
-				if (!qflag)
-					fprintf(stderr, "'%s' not found.\n", av);
-			}
-			continue;
-		}
-		if (lflag && !locatestring(path, localprefix, MATCH_AT_FIRST))
-			continue;
-		/*
-		 * Add a path to the path list.
-		 */
-		strbuf_puts0(path_list, path);
-		file_count++;
-	}
-	if (file_count > 0) {
-		/*
-		 * Execute parser in the root directory of source tree.
-		 */
-		if (chdir(root) < 0)
-			die("cannot move to '%s' directory.", root);
-		xp = xargs_open_with_strbuf(strbuf_value(comline), 0, path_list);
-		if (format == FORMAT_PATH) {
-			SPLIT ptable;
-			char curpath[MAXPATHLEN+1];
-
-			curpath[0] = '\0';
-			while ((ctags_x = xargs_read(xp)) != NULL) {
-				if (split((char *)ctags_x, 4, &ptable) < 4) {
-					recover(&ptable);
-					die("too small number of parts.\n'%s'", ctags_x);
-				}
-				if (strcmp(curpath, ptable.part[PART_PATH].start)) {
-					strlimcpy(curpath, ptable.part[PART_PATH].start, sizeof(curpath));
-					convert_put(cv, curpath);
-					count++;
-				}
-			}
-		} else {
-			while ((ctags_x = xargs_read(xp)) != NULL) {
-				convert_put(cv, ctags_x);
-				count++;
-			}
-		}
-		xargs_close(xp);
-		if (chdir(cwd) < 0)
-			die("cannot move to '%s' directory.", cwd);
-	}
-	/*
-	 * Settlement
-	 */
-	gpath_close();
-	convert_close(cv);
-	strbuf_close(comline);
-	strbuf_close(path_list);
-	if (file_count == 0)
-		die("file not found.");
-	if (vflag) {
-		print_count(count);
-		fprintf(stderr, " (no index used).\n");
-	}
-}
 /*
  * parsefile_using_builtin_parser: parse file to pick up tags.
  *
@@ -1093,6 +967,7 @@ struct parsefile_data {
 	int target;
 	int extractmethod;
 	int count;
+	const char *fid;			/* fid of the file under processing */
 };
 static void
 put_syms(int type, const char *tag, int lno, const char *path, const char *line_image, void *arg)
@@ -1128,18 +1003,20 @@ put_syms(int type, const char *tag, int lno, const char *path, const char *line_
 		} else {
 			key = tag;
 		}
-		if (dbop_get(data->dbop, key) != NULL) {
-			if (!(data->target & TARGET_REF))
-				return;
-		} else {
-			if (!(data->target & TARGET_SYM))
-				return;
+		if (data->target == TARGET_REF || data->target == TARGET_SYM) {
+			if (dbop_get(data->dbop, key) != NULL) {
+				if (!(data->target & TARGET_REF))
+					return;
+			} else {
+				if (!(data->target & TARGET_SYM))
+					return;
+			}
 		}
 		break;
 	default:
 		return;
 	}
-	convert_put_using(data->cv, tag, path, lno, line_image);
+	convert_put_using(data->cv, tag, path, lno, line_image, data->fid);
 	data->count++;
 }
 void
@@ -1150,10 +1027,13 @@ parsefile_using_builtin_parser(char *const *argv, const char *cwd, const char *r
 	STRBUF *sb = strbuf_open(0);
 	char *langmap;
 	const char *plugin_parser, *av;
-	char path[MAXPATHLEN+1];
+	char path[MAXPATHLEN];
 	struct parsefile_data data;
 
-	data.target = 1 << db;
+	if (db == GRTAGS + GSYMS)
+		data.target = TARGET_REF|TARGET_SYM;
+	else
+		data.target = 1 << db;
 	data.extractmethod = getconfb("extractmethod");
 	if (getconfs("langmap", sb))
 		langmap = check_strdup(strbuf_value(sb));
@@ -1167,13 +1047,14 @@ parsefile_using_builtin_parser(char *const *argv, const char *cwd, const char *r
 	data.cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	if (gpath_open(dbpath, 0) < 0)
 		die("GPATH not found.");
-	if (data.target & (TARGET_REF | TARGET_SYM)) {
+	if (data.target == TARGET_REF || data.target == TARGET_SYM) {
 		data.dbop = dbop_open(makepath(dbpath, dbname(GTAGS), NULL), 0, 0, 0);
 		if (data.dbop == NULL)
 			die("%s not found.", dbname(GTAGS));
 	} else {
 		data.dbop = NULL;
 	}
+	data.fid = NULL;
 	/*
 	 * Execute parser in the root directory of source tree.
 	 */
@@ -1189,10 +1070,20 @@ parsefile_using_builtin_parser(char *const *argv, const char *cwd, const char *r
 		if (normalize(av, get_root_with_slash(), cwd, path, sizeof(path)) == NULL)
 			if (!qflag)
 				fprintf(stderr, "'%s' is out of source tree.\n", path + 2);
-		if (!gpath_path2fid(path, NULL)) {
-			if (!qflag)
-				fprintf(stderr, "'%s' not found in GPATH.\n", path + 2);
-			continue;
+		/*
+		 * Memorize the file id of the path. This is used in put_syms().
+		 */
+		{
+			static char s_fid[MAXFIDLEN];
+			const char *p = gpath_path2fid(path, NULL);
+
+			if (!p) {
+				if (!qflag)
+					fprintf(stderr, "'%s' not found in GPATH.\n", path + 2);
+				continue;
+			}
+			strlimcpy(s_fid, p, sizeof(s_fid));
+			data.fid = s_fid;
 		}
 		if (!test("f", makepath(root, path, NULL))) {
 			if (test("d", NULL)) {
@@ -1255,7 +1146,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 	GTP *gtp;
 	int flags = 0;
 	STRBUF *sb = NULL, *ib = NULL;
-	char curpath[MAXPATHLEN+1], curtag[IDENTLEN+1];
+	char curpath[MAXPATHLEN], curtag[IDENTLEN];
 	FILE *fp = NULL;
 	const char *src = "";
 	int lineno, last_lineno;
@@ -1292,7 +1183,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 		if (lflag && !locatestring(gtp->path, localprefix, MATCH_AT_FIRST))
 			continue;
 		if (format == FORMAT_PATH) {
-			convert_put(cv, gtp->path);
+			convert_put_path(cv, gtp->path);
 			count++;
 		} else if (gtop->format & GTAGS_COMPACT) {
 			/*
@@ -1301,9 +1192,10 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 			 * tagline = <file id> <tag name> <line no>,...
 			 */
 			char *p = (char *)gtp->tagline;
-			const char *tagname;
+			const char *fid, *tagname;
 			int n = 0;
 
+			fid = p;
 			while (*p != ' ')
 				p++;
 			*p++ = '\0';			/* a */
@@ -1381,7 +1273,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 					}
 					if (gtop->format & GTAGS_COMPNAME)
 						tagname = (char *)uncompress(tagname, gtp->tag);
-					convert_put_using(cv, tagname, gtp->path, n, src);
+					convert_put_using(cv, tagname, gtp->path, n, src, fid);
 					count++;
 					last_lineno = last = n;
 				}
@@ -1411,7 +1303,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 					}
 					if (gtop->format & GTAGS_COMPNAME)
 						tagname = (char *)uncompress(tagname, gtp->tag);
-					convert_put_using(cv, tagname, gtp->path, n, src);
+					convert_put_using(cv, tagname, gtp->path, n, src, fid);
 					count++;
 					last_lineno = n;
 				}
@@ -1423,9 +1315,10 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 			 * tagline = <file id> <tag name> <line no> <line image>
 			 */
 			char *p = (char *)gtp->tagline;
-			char namebuf[IDENTLEN+1];
-			const char *tagname, *image;
+			char namebuf[IDENTLEN];
+			const char *fid, *tagname, *image;
 
+			fid = p;
 			while (*p != ' ')
 				p++;
 			*p++ = '\0';			/* a */
@@ -1446,7 +1339,7 @@ search(const char *pattern, const char *root, const char *cwd, const char *dbpat
 				if (gtop->format & GTAGS_COMPRESS)
 					image = (char *)uncompress(image, gtp->tag);
 			}
-			convert_put_using(cv, tagname, gtp->path, gtp->lineno, image);
+			convert_put_using(cv, tagname, gtp->path, gtp->lineno, image, fid);
 			count++;
 		}
 	}
@@ -1473,7 +1366,7 @@ void
 tagsearch(const char *pattern, const char *cwd, const char *root, const char *dbpath, int db)
 {
 	int count, total = 0;
-	char libdbpath[MAXPATHLEN+1];
+	char libdbpath[MAXPATHLEN];
 
 	/*
 	 * search in current source tree.
