@@ -59,9 +59,6 @@ static void usage(void);
 static void help(void);
 int main(int, char **);
 int incremental(const char *, const char *);
-void updatetags(const char *, const char *, IDSET *, STRBUF *, int);
-void createtags(const char *, const char *, int);
-static int peek_dbformat(const char *, const char *, int);
 void updatetags_using_builtin_parser(const char *, const char *, IDSET *, STRBUF *);
 void createtags_using_builtin_parser(const char *, const char *);
 int printconf(const char *);
@@ -73,7 +70,6 @@ int Oflag;					/* use objdir */
 int qflag;					/* quiet mode */
 int wflag;					/* warning message */
 int vflag;					/* verbose mode */
-int max_args;
 int show_version;
 int show_help;
 int show_config;
@@ -94,7 +90,6 @@ int convert_type = PATH_RELATIVE;
 
 int extractmethod;
 int total;
-int use_command_parser;				/* command layer parser is obsolete */
 
 static void
 usage(void)
@@ -146,7 +141,9 @@ static struct option const long_options[] = {
 #define OPT_GTAGSLABEL		130
 #define OPT_PATH		131
 #define OPT_SINGLE_UPDATE	132
+#define OPT_ENCODE_PATH		133
 	{"config", optional_argument, NULL, OPT_CONFIG},
+	{"encode-path", required_argument, NULL, OPT_ENCODE_PATH},
 	{"gtagsconf", required_argument, NULL, OPT_GTAGSCONF},
 	{"gtagslabel", required_argument, NULL, OPT_GTAGSLABEL},
 	{"path", required_argument, NULL, OPT_PATH},
@@ -155,14 +152,14 @@ static struct option const long_options[] = {
 };
 
 static const char *langmap = DEFAULTLANGMAP;
+static const char *gtags_parser;
 
 int
 main(int argc, char **argv)
 {
-	char dbpath[MAXPATHLEN+1];
-	char cwd[MAXPATHLEN+1];
+	char dbpath[MAXPATHLEN];
+	char cwd[MAXPATHLEN];
 	STRBUF *sb = strbuf_open(0);
-	int db;
 	int optchar;
 	int option_index = 0;
 	STATISTICS_TIME *tim;
@@ -198,6 +195,13 @@ main(int argc, char **argv)
 			iflag++;
 			single_update = optarg;
 			break;
+		case OPT_ENCODE_PATH:
+			if (strlen(optarg) > 255)
+				die("too many encode chars.");
+			if (strchr(optarg, '/') || strchr(optarg, '.'))
+				die("cannot encode '/' and '.' in the path.");
+			set_encode_chars((unsigned char *)optarg);
+			break;
 		case 'c':
 			cflag++;
 			break;
@@ -213,11 +217,6 @@ main(int argc, char **argv)
 		case 'I':
 			Iflag++;
 			break;
-		case 'n':
-			max_args = atoi(optarg);
-			if (max_args <= 0)
-				die("--max-args option requires number > 0.");
-			break;
 		case 'o':
 			/*
 			 * Though the -o(--omit-gsyms) was removed, this code
@@ -243,7 +242,7 @@ main(int argc, char **argv)
 		}
 	}
 	if (gtagsconf) {
-		char path[MAXPATHLEN+1];
+		char path[MAXPATHLEN];
 
 		if (realpath(gtagsconf, path) == NULL)
 			die("%s not found.", gtagsconf);
@@ -350,7 +349,7 @@ main(int argc, char **argv)
 	 * Regularize the path name for single updating (--single-update).
 	 */
 	if (single_update) {
-		static char regular_path_name[MAXPATHLEN+1];
+		static char regular_path_name[MAXPATHLEN];
 		char *p = single_update;
 		
 		if (!test("f", p))
@@ -393,9 +392,10 @@ main(int argc, char **argv)
 			strlimcpy(dbpath, cwd, sizeof(dbpath));
 	}
 	if (iflag && (!test("f", makepath(dbpath, dbname(GTAGS), NULL)) ||
+		!test("f", makepath(dbpath, dbname(GRTAGS), NULL)) ||
 		!test("f", makepath(dbpath, dbname(GPATH), NULL)))) {
 		if (wflag)
-			warning("GTAGS or GPATH not found. -i option ignored.");
+			warning("GTAGS, GRTAGS or GPATH not found. -i option ignored.");
 		iflag = 0;
 	}
 	if (!test("d", dbpath))
@@ -403,45 +403,23 @@ main(int argc, char **argv)
 	if (vflag)
 		fprintf(stderr, "[%s] Gtags started.\n", now());
 	/*
-	 * load .globalrc or /etc/gtags.conf
+	 * load configuration file.
 	 */
 	openconf();
 	if (getconfb("extractmethod"))
 		extractmethod = 1;
-	/*
-	 * Command layer parser is obsolete.
-	 */
-	if (getconfs("GTAGS", NULL))
-		use_command_parser = 1;
-	strbuf_reset(sb);
-	/*
-	 * Pass the following information to gtags-parser(1)
-	 * using environment variable.
-	 *
-	 * o langmap
-	 * o DBPATH
-	 */
 	strbuf_reset(sb);
 	if (getconfs("langmap", sb))
 		langmap = check_strdup(strbuf_value(sb));
-	if (!use_command_parser) {
-		const char *plugin_parser;
-
-		strbuf_reset(sb);
-		if (getconfs("gtags_parser", sb))
-			plugin_parser = strbuf_value(sb);
-		else
-			plugin_parser = NULL;
-		parser_init(langmap, plugin_parser);
-	} else {
-		/*
-		 * Command layer parser is obsolete.
-		 */
-		set_env("GTAGSLANGMAP", langmap);
-		set_env("GTAGSDBPATH", dbpath);
-		if (wflag)
-			set_env("GTAGSWARNING", "1");
-	}
+	strbuf_reset(sb);
+	if (getconfs("gtags_parser", sb))
+		gtags_parser = check_strdup(strbuf_value(sb));
+	/*
+	 * initialize parser.
+	 */
+	if (vflag && gtags_parser)
+		fprintf(stderr, " Using plug-in parser.\n");
+	parser_init(langmap, gtags_parser);
 	/*
 	 * incremental update.
 	 */
@@ -459,72 +437,14 @@ main(int argc, char **argv)
 		 */
 		if (!test("f", makepath(dbpath, dbname(GPATH), NULL)))
 			die("Old version tag file found. Please remake it.");
-		/*
-		 * The following restrictions are in incremental updating with
-		 * built-in parser:
-		 * If GRTAGS or GSYMS exists, both of them exist and the format
-		 * should be the same.
-		 */
-		if (!use_command_parser) {
-			int format_r, format_s;
-
-			if (test("f", makepath(dbpath, dbname(GRTAGS), NULL)))
-				format_r = peek_dbformat(dbpath, cwd, GRTAGS);
-			else
-				format_r = -1;
-			if (test("f", makepath(dbpath, dbname(GSYMS), NULL)))
-				format_s = peek_dbformat(dbpath, cwd, GSYMS);
-			else
-				format_s = -1;
-			if (format_r != format_s)
-				die("%s\n  Please invoke gtags again without the -i option.",
-					(format_r == -1) ? "GRTAGS doesn't exist though GSYMS exists."
-					: (format_s == -1) ? "GSYMS doesn't exist though GRTAGS exists."
-					: "The format of GRTAGS and GSYMS is different.");
-		}
 		(void)incremental(dbpath, cwd);
 		exit(0);
 	}
 	init_statistics();
 	/*
-	 * create GTAGS, GRTAGS and GSYMS
+	 * create GTAGS and GRTAGS
 	 */
-	if (!use_command_parser) {
-		createtags_using_builtin_parser(dbpath, cwd);
-	} else {
-		/*
-		 * Command layer parser is obsolete.
-		 */
-		for (db = GTAGS; db < GTAGLIM; db++) {
-			/*
-			 * get parser for db. (gtags-parser by default)
-			 */
-			strbuf_reset(sb);
-			if (!getconfs(dbname(db), sb))
-				continue;
-			if (!usable(strmake(strbuf_value(sb), " \t")))
-				die("Parser '%s' not found or not executable.", strmake(strbuf_value(sb), " \t"));
-			tim = statistics_time_start("Time of creating %s", dbname(db));
-			if (vflag)
-				fprintf(stderr, "[%s] Creating '%s'.\n", now(), dbname(db));
-			createtags(dbpath, cwd, db);
-			strbuf_reset(sb);
-			if (db == GTAGS) {
-				if (getconfs("GTAGS_extra", sb))
-					if (system(strbuf_value(sb)))
-						fprintf(stderr, "GTAGS_extra command failed: %s\n", strbuf_value(sb));
-			} else if (db == GRTAGS) {
-				if (getconfs("GRTAGS_extra", sb))
-					if (system(strbuf_value(sb)))
-						fprintf(stderr, "GRTAGS_extra command failed: %s\n", strbuf_value(sb));
-			} else if (db == GSYMS) {
-				if (getconfs("GSYMS_extra", sb))
-					if (system(strbuf_value(sb)))
-						fprintf(stderr, "GSYMS_extra command failed: %s\n", strbuf_value(sb));
-			}
-			statistics_time_end(tim);
-		}
-	}
+	createtags_using_builtin_parser(dbpath, cwd);
 	/*
 	 * create idutils index.
 	 */
@@ -548,11 +468,8 @@ main(int argc, char **argv)
 			fprintf(stderr, "executing mkid like: %s\n", strbuf_value(sb));
 		if (system(strbuf_value(sb)))
 			die("mkid failed: %s", strbuf_value(sb));
-		strbuf_reset(sb);
-		strbuf_puts(sb, "chmod 644 ");
-		strbuf_puts(sb, makepath(dbpath, "ID", NULL));
-		if (system(strbuf_value(sb)))
-			die("chmod failed: %s", strbuf_value(sb));
+		if (chmod(makepath(dbpath, "ID", NULL), 0644) < 0)
+			die("cannot chmod ID file.");
 		statistics_time_end(tim);
 	}
 	if (vflag)
@@ -676,7 +593,7 @@ normal_update:
 		 */
 		limit = gpath_nextkey();
 		for (id = 1; id < limit; id++) {
-			char fid[32];
+			char fid[MAXFIDLEN];
 			int type;
 
 			snprintf(fid, sizeof(fid), "%d", id);
@@ -706,26 +623,7 @@ normal_update:
 	 * execute updating.
 	 */
 	if (!idset_empty(deleteset) || strbuf_getlen(addlist) > 0) {
-		if (!use_command_parser) {
-			updatetags_using_builtin_parser(dbpath, root, deleteset, addlist);
-		} else {
-			/*
-			 * Command layer parser is obsolete.
-			 */
-			int db;
-
-			for (db = GTAGS; db < GTAGLIM; db++) {
-				/*
-				 * GTAGS needed at least.
-				 */
-				if ((db == GRTAGS || db == GSYMS)
-				    && !test("f", makepath(dbpath, dbname(db), NULL)))
-					continue;
-				if (vflag)
-					fprintf(stderr, "[%s] Updating '%s'.\n", now(), dbname(db));
-				updatetags(dbpath, root, deleteset, addlist, db);
-			}
-		}
+		updatetags_using_builtin_parser(dbpath, root, deleteset, addlist);
 		updated = 1;
 	}
 	if (strbuf_getlen(deletelist) + strbuf_getlen(addlist_other) > 0) {
@@ -775,242 +673,6 @@ normal_update:
 
 	return updated;
 }
-/*
- * updatetags: update tag file (using command layer parser)
- *
- *	i)	dbpath		directory in which tag file exist
- *	i)	root		root directory of source tree
- *	i)	deleteset	bit array of fid of deleted or modified files 
- *	i)	addlist		\0 separated list of added or modified files
- *	i)	db		GTAGS, GRTAGS, GSYMS
- *
- * Command layer parser is obsolete.
- */
-static void
-verbose_updatetags(char *path, int seqno, int skip)
-{
-	if (total)
-		fprintf(stderr, " [%d/%d]", seqno, total);
-	else
-		fprintf(stderr, " [%d]", seqno);
-	fprintf(stderr, " adding tags of %s", path);
-	if (skip)
-		fprintf(stderr, " (skipped)");
-	fputc('\n', stderr);
-}
-void
-updatetags(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addlist, int db)
-{
-	GTOP *gtop;
-	STRBUF *comline = strbuf_open(0);
-	int seqno;
-
-	/*
-	 * GTAGS needed to make GRTAGS.
-	 */
-	if (db == GRTAGS && !test("f", makepath(dbpath, dbname(GTAGS), NULL)))
-		die("GTAGS needed to create GRTAGS.");
-
-	/*
-	 * get tag command.
-	 */
-	if (!getconfs(dbname(db), comline))
-		die("cannot get tag command. (%s)", dbname(db));
-	gtop = gtags_open(dbpath, root, db, GTAGS_MODIFY, 0);
-	/*
-	 * Delete tags from GTAGS.
-	 */
-	if (!idset_empty(deleteset)) {
-		if (vflag) {
-			char fid[32];
-			const char *path;
-			int total = idset_count(deleteset);
-			unsigned int id;
-
-			seqno = 1;
-			for (id = idset_first(deleteset); id != END_OF_ID; id = idset_next(deleteset)) {
-				snprintf(fid, sizeof(fid), "%d", id);
-				path = gpath_fid2path(fid, NULL);
-				if (path == NULL)
-					die("GPATH is corrupted.");
-				fprintf(stderr, " [%d/%d] deleting tags of %s\n", seqno++, total, path + 2);
-			}
-		}
-		gtags_delete(gtop, deleteset);
-	}
-	gtop->flags = 0;
-	if (extractmethod)
-		gtop->flags |= GTAGS_EXTRACTMETHOD;
-	if (debug)
-		gtop->flags |= GTAGS_DEBUG;
-	/*
-	 * Compact format requires the tag records of the same file are
-	 * consecutive. We assume that the output of gtags-parser and
-	 * any plug-in parsers are consecutive for each file.
-	 * if (gtop->format & GTAGS_COMPACT) {
-	 *	nothing to do
-	 * }
-	 */
-	/*
-	 * If the --max-args option is not specified, we pass the parser
-	 * the source file as a lot as possible to decrease the invoking
-	 * frequency of the parser.
-	 */
-	{
-		XARGS *xp;
-		char *ctags_x;
-		char tag[MAXTOKEN], *p;
-
-		xp = xargs_open_with_strbuf(strbuf_value(comline), max_args, addlist);
-		xp->put_gpath = 1;
-		if (vflag)
-			xp->verbose = verbose_updatetags;
-		while ((ctags_x = xargs_read(xp)) != NULL) {
-			strlimcpy(tag, strmake(ctags_x, " \t"), sizeof(tag));
-			/*
-			 * extract method when class method definition.
-			 *
-			 * Ex: Class::method(...)
-			 *
-			 * key	= 'method'
-			 * data = 'Class::method  103 ./class.cpp ...'
-			 */
-			p = tag;
-			if (gtop->flags & GTAGS_EXTRACTMETHOD) {
-				if ((p = locatestring(tag, ".", MATCH_LAST)) != NULL)
-					p++;
-				else if ((p = locatestring(tag, "::", MATCH_LAST)) != NULL)
-					p += 2;
-				else
-					p = tag;
-			}
-			gtags_put(gtop, p, ctags_x);
-		}
-		total = xargs_close(xp);
-	}
-	gtags_close(gtop);
-	strbuf_close(comline);
-}
-/*
- * createtags: create tags file (using command layer parser)
- *
- *	i)	dbpath	dbpath directory
- *	i)	root	root directory of source tree
- *	i)	db	GTAGS, GRTAGS, GSYMS
- *
- * Command layer parser is obsolete.
- */
-static void
-verbose_createtags(char *path, int seqno, int skip)
-{
-	if (total)
-		fprintf(stderr, " [%d/%d]", seqno, total);
-	else
-		fprintf(stderr, " [%d]", seqno);
-	fprintf(stderr, " extracting tags of %s", path);
-	if (skip)
-		fprintf(stderr, " (skipped)");
-	fputc('\n', stderr);
-}
-void
-createtags(const char *dbpath, const char *root, int db)
-{
-	GTOP *gtop;
-	XARGS *xp;
-	char *ctags_x;
-	STRBUF *comline = strbuf_open(0);
-
-	/*
-	 * get tag command.
-	 */
-	if (!getconfs(dbname(db), comline))
-		die("cannot get tag command. (%s)", dbname(db));
-	/*
-	 * GTAGS needed to make GRTAGS.
-	 */
-	if (db == GRTAGS && !test("f", makepath(dbpath, dbname(GTAGS), NULL)))
-		die("GTAGS needed to create GRTAGS.");
-	gtop = gtags_open(dbpath, root, db, GTAGS_CREATE, cflag ? GTAGS_COMPACT : 0);
-	/*
-	 * Set flags.
-	 */
-	gtop->flags = 0;
-	if (extractmethod)
-		gtop->flags |= GTAGS_EXTRACTMETHOD;
-	if (debug)
-		gtop->flags |= GTAGS_DEBUG;
-	/*
-	 * Compact format requires the tag records of the same file are
-	 * consecutive. We assume that the output of gtags-parser and
-	 * any plug-in parsers are consecutive for each file.
-	 * if (gtop->format & GTAGS_COMPACT) {
-	 *	nothing to do
-	 * }
-	 */
-	/*
-	 * If the --max-args option is not specified, we pass the parser
-	 * the source file as a lot as possible to decrease the invoking
-	 * frequency of the parser.
-	 */
-	if (file_list)
-		find_open_filelist(file_list, root);
-	else
-		find_open(NULL);
-	/*
-	 * Add tags.
-	 */
-	xp = xargs_open_with_find(strbuf_value(comline), max_args);
-	xp->put_gpath = 1;
-	if (vflag)
-		xp->verbose = verbose_createtags;
-	while ((ctags_x = xargs_read(xp)) != NULL) {
-		char tag[MAXTOKEN], *p;
-
-		strlimcpy(tag, strmake(ctags_x, " \t"), sizeof(tag));
-		/*
-		 * extract method when class method definition.
-		 *
-		 * Ex: Class::method(...)
-		 *
-		 * key	= 'method'
-		 * data = 'Class::method  103 ./class.cpp ...'
-		 */
-		p = tag;
-		if (gtop->flags & GTAGS_EXTRACTMETHOD) {
-			if ((p = locatestring(tag, ".", MATCH_LAST)) != NULL)
-				p++;
-			else if ((p = locatestring(tag, "::", MATCH_LAST)) != NULL)
-				p += 2;
-			else
-				p = tag;
-		}
-		gtags_put(gtop, p, ctags_x);
-	}
-	total = xargs_close(xp);
-	find_close();
-	gtags_close(gtop);
-	strbuf_close(comline);
-}
-/*
- * peek_dbformat: peek format of tags file
- *
- *	i)	dbpath	dbpath directory
- *	i)	root	root directory of source tree
- *	i)	db	GTAGS, GRTAGS, GSYMS
- *	r)		db format
- */
-static int
-peek_dbformat(const char *dbpath, const char *root, int db)
-{
-	GTOP *gtop;
-	int format;
-
-	gtop = gtags_open(dbpath, root, db, GTAGS_READ, 0);
-	format = gtop->format;
-	gtags_close(gtop);
-
-	return format;
-}
 /*
  * callback functions for built-in parser
  */
@@ -1050,19 +712,30 @@ void
 updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addlist)
 {
 	struct put_func_data data;
-	int seqno;
+	int seqno, flags;
 	const char *path, *start, *end;
-	FILE *tmp;
 
 	if (vflag)
-		fprintf(stderr, "[%s] Updating '%s'.\n", now(), dbname(GTAGS));
+		fprintf(stderr, "[%s] Updating '%s' and '%s'.\n", now(), dbname(GTAGS), dbname(GRTAGS));
+	/*
+	 * Open tag files.
+	 */
 	data.gtop[GTAGS] = gtags_open(dbpath, root, GTAGS, GTAGS_MODIFY, 0);
+	if (test("f", makepath(dbpath, dbname(GRTAGS), NULL))) {
+		data.gtop[GRTAGS] = gtags_open(dbpath, root, GRTAGS, GTAGS_MODIFY, 0);
+	} else {
+		/*
+		 * If you set NULL to data.gtop[GRTAGS], parse_file() doesn't write to
+		 * GRTAGS. See put_syms().
+		 */
+		data.gtop[GRTAGS] = NULL;
+	}
 	/*
 	 * Delete tags from GTAGS.
 	 */
 	if (!idset_empty(deleteset)) {
 		if (vflag) {
-			char fid[32];
+			char fid[MAXFIDLEN];
 			int total = idset_count(deleteset);
 			unsigned int id;
 
@@ -1076,33 +749,20 @@ updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *del
 			}
 		}
 		gtags_delete(data.gtop[GTAGS], deleteset);
-	}
-	data.gtop[GTAGS]->flags = 0;
-	if (extractmethod)
-		data.gtop[GTAGS]->flags |= GTAGS_EXTRACTMETHOD;
-	if (debug)
-		data.gtop[GTAGS]->flags |= GTAGS_DEBUG;
-	if (test("f", makepath(dbpath, dbname(GRTAGS), NULL))) {
-		data.gtop[GRTAGS] = gtags_open(dbpath, root, GRTAGS, GTAGS_MODIFY, 0);
-		data.gtop[GRTAGS]->flags = data.gtop[GTAGS]->flags;
-		/*
-		 * If you set file pointer to tmpfile_fp, gtags_put() and gtags_put_using()
-		 * write records to the temporary file instead of db(3) file.
-		 */
-		tmp = tmpfile();
-		if (tmp == NULL)
-			die("cannot make temporary file.");
-		data.gtop[GRTAGS]->tmpfile_fp = tmp;
-	} else {
-		/*
-		 * If you set NULL to data.gtop[GRTAGS], parse_file() doesn't write to
-		 * GRTAGS. See put_syms().
-		 */
-		data.gtop[GRTAGS] = NULL;
-		tmp = NULL;
+		if (data.gtop[GRTAGS] != NULL)
+			gtags_delete(data.gtop[GRTAGS], deleteset);
 	}
 	/*
-	 * Add tags to GTAGS and the temporary file.
+	 * Set flags.
+	 */
+	flags = 0;
+	if (extractmethod)
+		flags |= GTAGS_EXTRACTMETHOD;
+	if (debug)
+		flags |= GTAGS_DEBUG;
+	data.gtop[GTAGS]->flags = data.gtop[GRTAGS]->flags = flags;
+	/*
+	 * Add tags to GTAGS and GRTAGS.
 	 */
 	start = strbuf_value(addlist);
 	end = start + strbuf_getlen(addlist);
@@ -1120,67 +780,9 @@ updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *del
 			gtags_flush(data.gtop[GRTAGS], data.fid);
 	}
 	parser_exit();
-	if (data.gtop[GRTAGS] == NULL) {
-		gtags_close(data.gtop[GTAGS]);
-		return;
-	}
-
-	if (vflag)
-		fprintf(stderr, "[%s] Updating '%s' and '%s'.\n", now(), dbname(GRTAGS), dbname(GSYMS));
-	data.gtop[GRTAGS]->tmpfile_fp = NULL;
-	data.gtop[GSYMS] = gtags_open(dbpath, root, GSYMS, GTAGS_MODIFY, 0);
-	/*
-	 * Delete tags from GRTAGS and GSYMS.
-	 */
-	if (!idset_empty(deleteset)) {
-		if (vflag) {
-			char fid[32];
-			int total = idset_count(deleteset);
-			unsigned int id;
-
-			seqno = 1;
-			for (id = idset_first(deleteset); id != END_OF_ID; id = idset_next(deleteset)) {
-				snprintf(fid, sizeof(fid), "%d", id);
-				path = gpath_fid2path(fid, NULL);
-				if (path == NULL)
-					die("GPATH is corrupted.");
-				fprintf(stderr, " [%d/%d] deleting tags of %s\n", seqno++, total, path + 2);
-			}
-		}
-		gtags_delete(data.gtop[GRTAGS], deleteset);
-		gtags_delete(data.gtop[GSYMS], deleteset);
-	}
-	/*
-	 * Move tags between GRTAGS and GSYMS.
-	 *
-	 * GRTAGS if not defined in GTAGS       ===> GSYMS
-	 * GSYMS if defined in GTAGS            ===> GRTAGS
-	 */
-	if (vflag) {
-		fprintf(stderr, " moving undefined symbols from '%s' to '%s'.\n", dbname(GRTAGS), dbname(GSYMS));
-		fprintf(stderr, " moving defined symbols from '%s' to '%s'.\n", dbname(GSYMS), dbname(GRTAGS));
-	}
-	gtags_move_ref_sym(data.gtop);
-	/*
-	 * Add tags to GRTAGS and GSYMS.
-	 *
-	 * temporary file ===>  if defined in GTAGS     ===> GRTAGS
-	 *                      else                    ===> GSYMS
-	 *
-	 * Gtags always makes GRTAGS and GSYMS even if they are empty.
-	 */
-	if (ftell(tmp) > 0) {
-		if (vflag) {
-			seqno = 0;
-			for (path = start; path < end; path += strlen(path) + 1)
-				fprintf(stderr, " [%d/%d] adding tags of %s\n", ++seqno, total, path + 2);
-		}
-		gtags_add_ref_sym(data.gtop, tmp);
-	}
-	fclose(tmp);
 	gtags_close(data.gtop[GTAGS]);
-	gtags_close(data.gtop[GRTAGS]);
-	gtags_close(data.gtop[GSYMS]);
+	if (data.gtop[GRTAGS] != NULL)
+		gtags_close(data.gtop[GRTAGS]);
 }
 /*
  * createtags_using_builtin_parser: create tags file
@@ -1196,14 +798,10 @@ createtags_using_builtin_parser(const char *dbpath, const char *root)
 	struct put_func_data data;
 	int openflags, seqno;
 	const char *path;
-	FILE *tmp;
 
-	tim = statistics_time_start("Time of creating %s and temporary file", dbname(GTAGS));
+	tim = statistics_time_start("Time of creating %s and %s.", dbname(GTAGS), dbname(GRTAGS));
 	if (vflag)
-		fprintf(stderr, "[%s] Creating '%s' and temporary file.\n", now(), dbname(GTAGS));
-	tmp = tmpfile();
-	if (tmp == NULL)
-		die("cannot make temporary file.");
+		fprintf(stderr, "[%s] Creating '%s' and '%s'.\n", now(), dbname(GTAGS), dbname(GRTAGS));
 	openflags = cflag ? GTAGS_COMPACT : 0;
 	data.gtop[GTAGS] = gtags_open(dbpath, root, GTAGS, GTAGS_CREATE, openflags);
 	data.gtop[GTAGS]->flags = 0;
@@ -1214,12 +812,7 @@ createtags_using_builtin_parser(const char *dbpath, const char *root)
 	data.gtop[GRTAGS] = gtags_open(dbpath, root, GRTAGS, GTAGS_CREATE, openflags);
 	data.gtop[GRTAGS]->flags = data.gtop[GTAGS]->flags;
 	/*
-	 * If you set file pointer to tmpfile_fp, gtags_put() and gtags_put_using()
-	 * write records to the temporary file instead of db(3) file.
-	 */
-	data.gtop[GRTAGS]->tmpfile_fp = tmp;
-	/*
-	 * Add tags to GTAGS and the temporary file.
+	 * Add tags to GTAGS and GRTAGS.
 	 */
 	if (file_list)
 		find_open_filelist(file_list, root);
@@ -1247,74 +840,25 @@ createtags_using_builtin_parser(const char *dbpath, const char *root)
 	total = seqno;
 	parser_exit();
 	find_close();
-	data.gtop[GRTAGS]->tmpfile_fp = NULL;
-	statistics_time_end(tim);
-	strbuf_reset(sb);
-	if (getconfs("GTAGS_extra", sb)) {
-		tim = statistics_time_start("Time of executing GTAGS_extra command");
-		/*
-		 * GTAGS_extra command may modify GTAGS.
-		 * Closing and reopening are necessary to avoid damaging GTAGS.
-		 */
-		dbop_close(data.gtop[GTAGS]->dbop);
-		if (system(strbuf_value(sb)))
-			fprintf(stderr, "GTAGS_extra command failed: %s\n", strbuf_value(sb));
-		data.gtop[GTAGS]->dbop = dbop_open(makepath(dbpath, dbname(GTAGS), NULL), 0, 0, 0);
-		if (data.gtop[GTAGS]->dbop == NULL)
-			die("%s not found.", dbname(GTAGS));
-		statistics_time_end(tim);
-	}
-
-	tim = statistics_time_start("Time of creating %s and %s", dbname(GRTAGS), dbname(GSYMS));
-	if (vflag)
-		fprintf(stderr, "[%s] Creating '%s' and '%s'.\n", now(), dbname(GRTAGS), dbname(GSYMS));
-	data.gtop[GSYMS] = gtags_open(dbpath, root, GSYMS, GTAGS_CREATE, openflags);
-	/*
-	 * Add tags to GRTAGS and GSYMS.
-	 *
-	 * temporary file ===>  if defined in GTAGS     ===> GRTAGS
-	 *                      else                    ===> GSYMS
-	 *
-	 * Gtags always makes GRTAGS and GSYMS even if they are empty.
-	 */
-	if (ftell(tmp) > 0) {
-		if (vflag) {
-			if (file_list)
-				find_open_filelist(file_list, root);
-			else
-				find_open(NULL);
-			seqno = 0;
-			while ((path = find_read()) != NULL) {
-				if (*path == ' ')
-					continue;
-				fprintf(stderr, " [%d/%d] adding tags of %s\n", ++seqno, total, path + 2);
-			}
-			find_close();
-		}
-		gtags_add_ref_sym(data.gtop, tmp);
-	}
-	fclose(tmp);
 	statistics_time_end(tim);
 	tim = statistics_time_start("Time of flushing B-tree cache");
 	gtags_close(data.gtop[GTAGS]);
 	gtags_close(data.gtop[GRTAGS]);
-	gtags_close(data.gtop[GSYMS]);
 	statistics_time_end(tim);
 	strbuf_reset(sb);
+	if (getconfs("GTAGS_extra", sb)) {
+		tim = statistics_time_start("Time of executing GTAGS_extra command");
+		if (system(strbuf_value(sb)))
+			fprintf(stderr, "GTAGS_extra command failed: %s\n", strbuf_value(sb));
+		statistics_time_end(tim);
+	}
+	strbuf_reset(sb);
 	if (getconfs("GRTAGS_extra", sb)) {
 		tim = statistics_time_start("Time of executing GRTAGS_extra command");
 		if (system(strbuf_value(sb)))
 			fprintf(stderr, "GRTAGS_extra command failed: %s\n", strbuf_value(sb));
 		statistics_time_end(tim);
 	}
-	strbuf_reset(sb);
-	if (getconfs("GSYMS_extra", sb)) {
-		tim = statistics_time_start("Time of executing GSYMS_extra command");
-		if (system(strbuf_value(sb)))
-			fprintf(stderr, "GSYMS_extra command failed: %s\n", strbuf_value(sb));
-		statistics_time_end(tim);
-	}
-
 	strbuf_close(sb);
 }
 /*
