@@ -95,8 +95,12 @@ const char *reserved_begin	= "<b>";
 const char *reserved_end	= "</b>";
 const char *position_begin	= "<font color='gray'>";
 const char *position_end	= "</font>";
-const char *warned_line_begin	= "<span style='background-color:yellow'>";
-const char *warned_line_end	= "</span>";
+const char *warned_line_begin	= "<font color='red'>";
+const char *warned_line_end	= "</font>";
+const char *current_line_begin	= "";
+const char *current_line_end	= "";
+const char *current_row_begin	= "<tr>";
+const char *current_row_end	= "</tr>";
 const char *error_begin		= "<h1><font color='#cc0000'>";
 const char *error_end		= "</font></h1>";
 const char *message_begin	= "<h3>";
@@ -189,6 +193,10 @@ setup_xhtml(void)
 	position_end		= "</em>";
 	warned_line_begin	= "<em class='warned'>";
 	warned_line_end		= "</em>";
+	current_line_begin	= "<span class='curline'>";
+	current_line_end	= "</span>";
+	current_row_begin	= "<tr class='curline'>";
+	current_row_end		= "</tr>";
 	error_begin		= "<h2 class='error'>";
 	error_end		= "</h2>";
 	message_begin		= "<h3 class='message'>";
@@ -208,9 +216,9 @@ setup_xhtml(void)
 /*
  * These methods is used to tell lex() the current path infomation.
  */
-static char current_path[MAXPATHLEN+1];
-static char current_dir[MAXPATHLEN+1];
-static char current_file[MAXPATHLEN+1];
+static char current_path[MAXPATHLEN];
+static char current_dir[MAXPATHLEN];
+static char current_file[MAXPATHLEN];
 /*
  * save path infomation
  */
@@ -342,8 +350,19 @@ static const char *
 gen_page_generic_begin(const char *title, int place, int use_frameset)
 {
 	STATIC_STRBUF(sb);
-	const char *dir = (place == SUBDIR) ? "../" : "";
+	const char *dir = NULL;
 
+	switch (place) {
+	case TOPDIR:
+		dir = "";
+		break;
+	case SUBDIR:
+		 dir = "../";
+		break;
+	case CGIDIR:
+		 dir = "$basedir/";	/* decided by the CGI script */
+		break;
+	}
 	strbuf_clear(sb);
 	if (enable_xhtml) {
 		/*
@@ -592,13 +611,14 @@ gen_list_begin(void)
 /*
  * Generate list body.
  *
- * s must be choped.
+ * ctags_x with the --encode-path=" \t"
  */
 const char *
-gen_list_body(const char *srcdir, const char *ctags_x)		/* virtually const */
+gen_list_body(const char *srcdir, const char *ctags_x, const char *fid)	/* virtually const */
 {
 	STATIC_STRBUF(sb);
-	const char *p, *filename, *fid;
+	char path[MAXPATHLEN];
+	const char *p;
 	SPLIT ptable;
 
 	strbuf_clear(sb);
@@ -606,25 +626,26 @@ gen_list_body(const char *srcdir, const char *ctags_x)		/* virtually const */
 		recover(&ptable);
 		die("too small number of parts in list_body().\n'%s'", ctags_x);
 	}
-	filename = ptable.part[PART_PATH].start + 2;	/* remove './' */
-	fid = path2fid(filename);
+	strlimcpy(path, decode_path((unsigned char *)ptable.part[PART_PATH].start + 2), sizeof(path));
+	if (fid == NULL)
+		fid = path2fid(path);
 	if (table_list) {
+		strbuf_puts(sb, current_row_begin);
 		if (enable_xhtml) {
-			strbuf_puts(sb, "<tr class='curline'><td class='tag'>");
+			strbuf_puts(sb, "<td class='tag'>");
 			strbuf_puts(sb, gen_href_begin(srcdir, fid, HTML, ptable.part[PART_LNO].start));
 			strbuf_puts(sb, ptable.part[PART_TAG].start);
 			strbuf_puts(sb, gen_href_end());
 			strbuf_sprintf(sb, "</td><td class='line'>%s</td><td class='file'>%s</td><td class='code'>",
-				ptable.part[PART_LNO].start, filename);
+				ptable.part[PART_LNO].start, path);
 		} else {
-			strbuf_puts(sb, "<tr class='curline'><td nowrap>");
+			strbuf_puts(sb, "<td nowrap>");
 			strbuf_puts(sb, gen_href_begin(srcdir, fid, HTML, ptable.part[PART_LNO].start));
 			strbuf_puts(sb, ptable.part[PART_TAG].start);
 			strbuf_puts(sb, gen_href_end());
 			strbuf_sprintf(sb, "</td><td nowrap align='right'>%s</td><td nowrap align='left'>%s</td><td nowrap>",
-				ptable.part[PART_LNO].start, filename);
+				ptable.part[PART_LNO].start, path);
 		}
-
 		for (p = ptable.part[PART_LINE].start; *p; p++) {
 			unsigned char c = *p;
 
@@ -642,26 +663,27 @@ gen_list_body(const char *srcdir, const char *ctags_x)		/* virtually const */
 			} else
 				strbuf_putc(sb, c);
 		}
-		strbuf_puts(sb, "</td></tr>");
+		strbuf_puts(sb, "</td>");
+		strbuf_puts(sb, current_row_end);
 		recover(&ptable);
 	} else {
-		int done = 0;
-
-		strbuf_puts(sb, "<span class='curline'>");
+		/* print tag name with anchor */
+		strbuf_puts(sb, current_line_begin);
 		strbuf_puts(sb, gen_href_begin(srcdir, fid, HTML, ptable.part[PART_LNO].start));
 		strbuf_puts(sb, ptable.part[PART_TAG].start);
 		strbuf_puts(sb, gen_href_end());
-		p = ctags_x + strlen(ptable.part[PART_TAG].start);
 		recover(&ptable);
 
-		for (; *p; p++) {
+		/* print line number */
+		for (p = ptable.part[PART_TAG].end; p < ptable.part[PART_PATH].start; p++)
+			strbuf_putc(sb, *p);
+		/* print file name */
+		strbuf_puts(sb, path);
+		/* print the rest */
+		for (p = ptable.part[PART_PATH].end; *p; p++) {
 			unsigned char c = *p;
 
-			/* ignore "./" in path name */
-			if (!done && c == '.' && *(p + 1) == '/') {
-				p++;
-				done = 1;
-			} else if (c == '&')
+			if (c == '&')
 				strbuf_puts(sb, quote_amp);
 			else if (c == '<')
 				strbuf_puts(sb, quote_little);
@@ -670,7 +692,7 @@ gen_list_body(const char *srcdir, const char *ctags_x)		/* virtually const */
 			else
 				strbuf_putc(sb, c);
 		}
-		strbuf_puts(sb, "</span>");
+		strbuf_puts(sb, current_line_end);
 	}
 	return strbuf_value(sb);
 }
