@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Tama Communications Corporation
+ * Copyright (c) 2004, 2008 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -27,8 +27,9 @@
 #define PARENT 1
 
 /* gen_page_begin() */
-#define SUBDIR 1
 #define TOPDIR 0
+#define SUBDIR 1
+#define CGIDIR 2
 
 /*
  * tag
@@ -99,6 +100,9 @@ extern const char *noframes_end;
 
 int fputs_nl(const char *, FILE *);
 void setup_xhtml(void);
+void save_current_path(const char *);
+char *get_current_dir();
+char *get_current_file();
 const char *upperdir(const char *);
 const char *gen_insert_header(int);
 const char *gen_insert_footer(int);
@@ -114,7 +118,7 @@ const char *gen_href_begin(const char *, const char *, const char *, const char
 const char *gen_href_begin_simple(const char *);
 const char *gen_href_end(void);
 const char *gen_list_begin(void);
-const char *gen_list_body(const char *, const char *);
+const char *gen_list_body(const char *, const char *, const char *);
 const char *gen_list_end(void);
 const char *gen_div_begin(const char *);
 const char *gen_div_end(void);
