@@ -343,7 +343,7 @@ print_directory(int level, char *basedir)
 	int count = 0;
 
 	if (level > 0) {
-		char name[MAXPATHLEN+1];
+		char name[MAXPATHLEN];
 
 		snprintf(name, sizeof(name), "%s/files/%s.%s", distpath, path2fid(basedir), HTML);
 		fileop = open_output_file(name, cflag);
@@ -445,7 +445,7 @@ print_directory_header(FILE *op, int level, const char *dir)
  	strbuf_sprintf(sb, "%s%sroot%s/", header_begin, gen_href_begin(NULL, indexlink, normal_suffix, NULL), gen_href_end());
 	fputs(strbuf_value(sb), op);
 	{
-		char path[MAXPATHLEN+1];
+		char path[MAXPATHLEN];
 		char *p, *q;
 
 		strlimcpy(path, dir, sizeof(path));
@@ -469,7 +469,7 @@ print_directory_header(FILE *op, int level, const char *dir)
 	}
 	fputs_nl(header_end, op);
 	{
-		char parentdir[MAXPATHLEN+1];
+		char parentdir[MAXPATHLEN];
 		const char *suffix, *parent;
 
 		(void)dirpart(dir, parentdir);
@@ -507,7 +507,7 @@ static void
 print_directory_footer(FILE *op, int level, const char *dir)
 {
 	const char *parent, *suffix;
-	char parentdir[MAXPATHLEN+1];
+	char parentdir[MAXPATHLEN];
 
 	if (level == 0)
 		die("print_directory_footer: internal error.");
@@ -653,7 +653,7 @@ makefileindex(const char *file, STRBUF *a_files)
 	 * progress of processing. It isn't copied each every recursive call
 	 * not to waste the stack.
 	 */
-	char basedir[MAXPATHLEN+1];
+	char basedir[MAXPATHLEN];
 
 	/*
 	 * Initialize data.
@@ -717,6 +717,7 @@ makefileindex(const char *file, STRBUF *a_files)
 	if (map_file)
 		fclose(FILEMAP);
 	gfind_close(gp);
+	regfree(&is_include_file);
 
 	fputs(strbuf_value(files), filesop);
 	if (table_flist)
@@ -754,7 +755,7 @@ makeincludeindex(void)
 	 * Unlike Perl regular expression, POSIX regular expression doesn't support C-style escape sequence.
 	 * Therefore, we can not use "\\t" here.
 	 */
-	snprintf(command, sizeof(command), "%s -gnx \"^[ \t]*(#[ \t]*(import|include)|include[ \t]*\\()\"", global_path);
+	snprintf(command, sizeof(command), "%s -gnx --encode-path=\" \t\" \"^[ \t]*(#[ \t]*(import|include)|include[ \t]*\\()\"", global_path);
 	if ((PIPE = popen(command, "r")) == NULL)
 		die("cannot fork.");
 	strbuf_reset(input);
@@ -827,10 +828,11 @@ makeincludeindex(void)
 			close_file(fileop_INCLUDE);
 			html_count++;
 			/*
-			 * inc->path == NULL means that information already
+			 * inc->contents == NULL means that information already
 			 * written to file.
 			 */
-			strbuf_reset(inc->contents);
+			strbuf_close(inc->contents);
+			inc->contents = NULL;
 		}
 		if (!inc->ref_count)
 			continue;
@@ -842,7 +844,7 @@ makeincludeindex(void)
 				recover(&ptable);
 				die("too small number of parts in makefileindex().");
 			}
-			snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
+			snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, decode_path((unsigned char *)ptable.part[PART_PATH].start));
 			recover(&ptable);
 			strbuf_reset(inc->ref_contents);
 			strbuf_puts(inc->ref_contents, buf);
@@ -860,7 +862,7 @@ makeincludeindex(void)
 				int count = inc->ref_count;
 
 				for (; count; line += strlen(line) + 1, count--)
-					fputs_nl(gen_list_body(upperdir(SRCS), line), INCLUDE);
+					fputs_nl(gen_list_body(upperdir(SRCS), line, NULL), INCLUDE);
 			}
 			fputs_nl(gen_list_end(), INCLUDE);
 			fputs_nl(body_end, INCLUDE);
@@ -868,10 +870,11 @@ makeincludeindex(void)
 			close_file(fileop_INCLUDE);
 			html_count++;
 			/*
-			 * inc->path == NULL means that information already
+			 * inc->ref_contents == NULL means that information already
 			 * written to file.
 			 */
-			strbuf_reset(inc->ref_contents);
+			strbuf_close(inc->ref_contents);
+			inc->ref_contents = NULL;
 		}
 	}
 	strbuf_close(input);
