@@ -122,7 +122,7 @@ int statistics = STATISTICS_STYLE_NONE;	/* --statistics option		*/
 int no_order_list;			/* 1: doesn't use order list	*/
 int other_files;			/* 1: list other files		*/
 int enable_grep = 1;			/* 1: enable grep		*/
-int enable_idutils;			/* 1: enable idutils		*/
+int enable_idutils = 1;			/* 1: enable idutils		*/
 int enable_xhtml = 1;			/* 1: enable XHTML		*/
 
 const char *action_value;
@@ -276,6 +276,7 @@ static struct option const long_options[] = {
         {"caution", no_argument, &caution, 1},
         {"debug", no_argument, &debug, 1},
         {"disable-grep", no_argument, &enable_grep, 0},
+        {"disable-idutils", no_argument, &enable_idutils, 0},
         {"full-path", no_argument, &full_path, 1},
         {"html", no_argument, &enable_xhtml, 0},
         {"nocgi", no_argument, &cgi, 0},
@@ -311,7 +312,7 @@ static struct option const long_options[] = {
         {"insert-footer", required_argument, NULL, OPT_INSERT_FOOTER},
         {"insert-header", required_argument, NULL, OPT_INSERT_HEADER},
         {"item-order", required_argument, NULL, OPT_ITEM_ORDER},
-        {"tabs", required_argument, NULL, OPT_TABS},
+	{"tabs", required_argument, NULL, OPT_TABS},
         { 0 }
 };
 
@@ -394,9 +395,10 @@ make_directory_in_distpath(const char *name)
  *
  *	i)	dist	directory where the file should be created
  *	i)	file	file name
+ *	i)	place	TOPDIR, SUBDIR, CGIDIR
  */
 static void
-generate_file(const char *dist, const char *file)
+generate_file(const char *dist, const char *file, int place)
 {
 	regex_t preg;
 	regmatch_t pmatch[2];
@@ -434,7 +436,7 @@ generate_file(const char *dist, const char *file)
         };
 	int tabsize = sizeof(tab) / sizeof(struct map);
 
-	tab[0].value = gen_page_begin("Result", SUBDIR);
+	tab[0].value = gen_page_begin("Result", place);
 	tab[1].value = gen_page_end();
 	/*
 	 * construct regular expression.
@@ -516,7 +518,7 @@ generate_file(const char *dist, const char *file)
 static void
 makeprogram(const char *cgidir, const char *file)
 {
-	generate_file(cgidir, file);
+	generate_file(cgidir, file, CGIDIR);
 }
 /*
  * makebless: make bless.sh file.
@@ -526,7 +528,7 @@ makebless(const char *file)
 {
 	const char *save = action;
 	action = saction;
-	generate_file(distpath, file);
+	generate_file(distpath, file, SUBDIR);
 	action = save;
 }
 /*
@@ -538,7 +540,7 @@ makebless(const char *file)
 static void
 makeghtml(const char *cgidir, const char *file)
 {
-	generate_file(cgidir, file);
+	generate_file(cgidir, file, SUBDIR);
 }
 /*
  * makerebuild: make rebuild script
@@ -664,10 +666,8 @@ makesearchpart(const char *action, const char *id, const char *target)
 	strbuf_puts_nl(sb, target ? "Def" : "Definition");
 	strbuf_puts(sb, gen_input_radio("type", "reference", 0, "Retrieve the reference place of the specified symbol."));
 	strbuf_puts_nl(sb, target ? "Ref" : "Reference");
-	if (test("f", makepath(dbpath, dbname(GSYMS), NULL))) {
-		strbuf_puts(sb, gen_input_radio("type", "symbol", 0, "Retrieve the place of the specified symbol is used."));
-		strbuf_puts_nl(sb, target ? "Sym" : "Other symbol");
-	}
+	strbuf_puts(sb, gen_input_radio("type", "symbol", 0, "Retrieve the place of the specified symbol is used."));
+	strbuf_puts_nl(sb, target ? "Sym" : "Other symbol");
 	strbuf_puts(sb, gen_input_radio("type", "path", 0, "Look for path name which matches to the specified pattern."));
 	strbuf_puts(sb, target ? "Path" : "Path name");
 	if (enable_grep) {
@@ -818,7 +818,7 @@ makehtaccess(const char *file)
 	fputs_nl("# |AllowOverride Options FileInfo", op);
 	fputs_nl("#", op);
 	fputs_nl("# Htags was invoked with the -f, -c or -D option.", op);
-	fprintf(op, "# You should start HTTP server so that %s/cgi-bin/*.cgi is executed", cgidir);
+	fprintf(op, "# You should start HTTP server so that %s/*.cgi is executed\n", cgidir);
 	fputs_nl("# as a CGI script.", op);
 	fputs_nl("#", op);
 	fputs_nl("Options +ExecCGI", op);
@@ -998,13 +998,16 @@ makecommonpart(const char *title, const char *defines, const char *files)
 		case 'm':
 			strbuf_sprintf(sb, "%sMAINS%s\n", header_begin, header_end);
 
-			snprintf(buf, sizeof(buf), "%s -x --nofilter=path %s", global_path, main_func);
+			snprintf(buf, sizeof(buf), "%s --result=ctags-xid --encode-path=\" \t\" --nofilter=path %s", global_path, main_func);
 			ip = popen(buf, "r");
 			if (!ip)
 				die("cannot execute command '%s'.", buf);
 			strbuf_puts_nl(sb, gen_list_begin());
 			while ((_ = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
-				strbuf_puts_nl(sb, gen_list_body(SRCS, _));
+				char fid[MAXFIDLEN];
+				const char *ctags_x = parse_xid(_, fid, NULL);
+
+				strbuf_puts_nl(sb, gen_list_body(SRCS, ctags_x, fid));
 			}
 			strbuf_puts_nl(sb, gen_list_end());
 			if (pclose(ip) != 0)
@@ -1158,60 +1161,28 @@ configuration(int argc, char *const *argv)
 		else
 			tabs = n;
 	}
-	if (getconfn("flist_fields", &n)) {
-		if (n < 1)
-			warning("parameter 'flist_fields' ignored because the value (=%d) is too large or too small.", n);
-		else
-			flist_fields = n;
-	}
 	strbuf_reset(sb);
 	if (getconfs("gzipped_suffix", sb))
 		gzipped_suffix = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
 	if (getconfs("normal_suffix", sb))
 		normal_suffix = check_strdup(strbuf_value(sb));
-	strbuf_reset(sb);
-	if (getconfs("definition_header", sb)) {
-		p = strbuf_value(sb);
-		if (!strcmp(p, "no"))
-			definition_header = NO_HEADER;
-		else if (!strcmp(p, "before"))
-			definition_header = BEFORE_HEADER;
-		else if (!strcmp(p, "right"))
-			definition_header = RIGHT_HEADER;
-		else if (!strcmp(p, "after"))
-			definition_header = AFTER_HEADER;
-	}
-	if (getconfb("other_files"))
-		other_files = 1;
-	if (getconfb("disable_grep"))
-		enable_grep = 0;
-	if (getconfb("enable_idutils"))
-		enable_idutils = 1;
-	if (getconfb("full_path"))
-		full_path = 1;
-	if (getconfb("table_list"))
-		table_list = 1;
-	if (getconfb("table_flist"))
-		table_flist = 1;
 	if (getconfb("no_order_list"))
 		no_order_list = 1;
-	if (getconfb("no_map_file"))
-		map_file = 0;
+/*
 	strbuf_reset(sb);
 	if (getconfs("icon_spec", sb))
 		icon_spec = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
 	if (getconfs("icon_suffix", sb))
 		icon_suffix = check_strdup(strbuf_value(sb));
+*/
 	strbuf_reset(sb);
 	if (getconfs("prolog_script", sb))
 		prolog_script = check_strdup(strbuf_value(sb));
 	strbuf_reset(sb);
 	if (getconfs("epilog_script", sb))
 		epilog_script = check_strdup(strbuf_value(sb));
-	if (getconfb("show_position"))
-		show_position = 1;
 	if (getconfb("colorize_warned_line"))
 		colorize_warned_line = 1;
 	strbuf_reset(sb);
@@ -1223,8 +1194,6 @@ configuration(int argc, char *const *argv)
 			*q = '\0';
 		script_alias = p;
 	}
-	if (getconfb("dynamic"))
-		dynamic = 1;
 	strbuf_reset(sb);
 	if (getconfs("body_begin", sb)) {
 		p = check_strdup(strbuf_value(sb));
@@ -1693,7 +1662,7 @@ main(int argc, char **argv)
 			strbuf_puts(sb, " -v");
 		if (wflag)
 			strbuf_puts(sb, " -w");
-		if (enable_idutils)
+		if (enable_idutils && usable("mkid"))
 			strbuf_puts(sb, " -I");
 		if (arg_dbpath[0]) {
 			strbuf_putc(sb, ' ');
@@ -1772,12 +1741,12 @@ main(int argc, char **argv)
 			path = makepath(dbpath, dbname(i), NULL);
 			gtags_exist[i] = test("fr", path);
 		}
-		if (!gtags_exist[GPATH] || !gtags_exist[GTAGS])
-			die("GPATH and/or GTAGS not found. Please reexecute htags with the -g option.");
-		if (!symbol)
-			gtags_exist[GSYMS] = 0;
-		else if (!gtags_exist[GSYMS])
-			die("the -s(--symbol) option needs GSYMS. Please reexecute htags with the -g option.");
+		/*
+		 * Real GRTAGS includes virtual GSYMS.
+		 */
+		gtags_exist[GSYMS] = symbol ? 1 : 0;
+		if (!gtags_exist[GPATH] || !gtags_exist[GTAGS] || !gtags_exist[GRTAGS])
+			die("GPATH, GTAGS and/or GRTAGS not found. Please reexecute htags with the -g option.");
 		/*
 		 * version check.
 		 * Do nothing, but the version of tag file will be checked.
@@ -1860,6 +1829,8 @@ main(int argc, char **argv)
 	 */
 	if (get_dbpath())
 		message(" Using %s/GTAGS", get_dbpath());
+	if (gpath_open(get_dbpath(), 0) < 0)
+		die("GPATH not found.");
 	if (!w32) {
 		/* UNDER CONSTRUCTION */
 	}
@@ -1915,6 +1886,17 @@ main(int argc, char **argv)
 	} else {
 		message("[%s] (1) making CGI program ...(skipped)", now());
 	}
+	/*
+	 * Save the suffix of compress format for the safe CGI script.
+	 */
+	{
+		FILE *op = fopen(makepath(distpath, "compress", NULL), "w");
+		if (cflag) {
+			fputs(HTML, op);
+			fputc('\n', op);
+		}
+		fclose(op);
+	}
 	/*
 	 * (2) make help file
 	 */
@@ -2028,6 +2010,7 @@ main(int argc, char **argv)
 		snprintf(dst, sizeof(dst), "%s/style.css", distpath);
 		copyfile(src, dst);
 	}
+	message("[%s] Done.", now());
 	if (vflag && cgi && (cflag || fflag || dynamic)) {
 		message("\n[Information]\n");
 		if (fflag || cflag || dynamic) {
@@ -2053,6 +2036,7 @@ main(int argc, char **argv)
 			copyfile(src, dst);
 		}
 	}
+	gpath_close();
 	/*
 	 * Print statistics information.
 	 */
