@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *		2006, 2008
+ *		2006, 2008, 2010
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -355,12 +355,12 @@ put_anchor(char *name, int type, int lineno)
 		strbuf_puts(outbuf, name);
 	} else {
 		/*
-		 * About cache record format, please see the comment in cache.c.
+		 * About the format of 'line', please see the head comment of cache.c.
 		 */
 		if (*line == ' ') {
 			char tmp[MAXPATHLEN];
-			const char *id = strmake(++line, " ");
-			const char *count = locatestring(line, " ", MATCH_FIRST) + 1;
+			const char *id = line + 1;
+			const char *count = nextstring(id);
 			const char *dir, *file, *suffix = NULL;
 
 			if (dynamic) {
@@ -390,9 +390,11 @@ put_anchor(char *name, int type, int lineno)
 			strbuf_puts(outbuf, name);
 			strbuf_puts(outbuf, gen_href_end());
 		} else {
-			char lno[32];
-			const char *filename;
+			const char *lno = line;
+			const char *fid = nextstring(line);
+			const char *path = gpath_fid2path(fid, NULL);
 
+			path += 2;              /* remove './' */
 			/*
 			 * Don't make a link which refers to itself.
 			 * Being used only once means that it is a self link.
@@ -401,10 +403,7 @@ put_anchor(char *name, int type, int lineno)
 				strbuf_puts(outbuf, name);
 				return;
 			}
-			strlimcpy(lno, strmake(line, " "), sizeof(lno));
-			filename = strmake(locatestring(line, " ", MATCH_FIRST) + 1, " ")
-						+ 2;	/* remove './' */
-			strbuf_puts(outbuf, gen_href_begin_with_title(upperdir(SRCS), path2fid(filename), HTML, lno, tooltip(type, atoi(lno), filename)));
+			strbuf_puts(outbuf, gen_href_begin_with_title(upperdir(SRCS), fid, HTML, lno, tooltip(type, atoi(lno), path)));
 			strbuf_puts(outbuf, name);
 			strbuf_puts(outbuf, gen_href_end());
 		}
@@ -463,7 +462,7 @@ put_macro(const char *word)
 	strbuf_puts(outbuf, sharp_end);
 }
 /*
- * Print warning message when unkown preprocessing directive is found.
+ * Print warning message when unknown preprocessing directive is found.
  */
 void
 unknown_preprocessing_directive(const char *word, int lineno)
@@ -638,7 +637,7 @@ get_cvs_module(const char *file, const char **basename)
 {
 	const char *p;
 	STATIC_STRBUF(dir);
-	static char prev_dir[MAXPATHLEN+1];
+	static char prev_dir[MAXPATHLEN];
 	STATIC_STRBUF(module);
 	FILE *ip;
 
