@@ -51,11 +51,17 @@ typedef	struct {
 	 * (2) DB185 PART
 	 */
 	DB *db;				/* descripter of DB */
-	char dbname[MAXPATHLEN+1];	/* dbname */
-	char key[MAXKEYLEN+1];		/* key */
+	char dbname[MAXPATHLEN];	/* dbname */
+	char key[MAXKEYLEN];		/* key */
 	int keylen;			/* key length */
-	char prev[MAXKEYLEN+1];		/* previous key value */
+	char prev[MAXKEYLEN];		/* previous key value */
 	int perm;			/* file permission */
+	/*
+	 * (3) sorted write
+	 */
+	FILE *sortout;			/* write to sort command */
+	FILE *sortin;			/* read from sort command */
+	int pid;			/* sort process id */
 } DBOP;
 
 /*
@@ -65,9 +71,10 @@ typedef	struct {
 /*
  * ioflags
  */
-#define DBOP_KEY	1		/* read key part		*/
-#define DBOP_PREFIX	2		/* prefixed read		*/
-#define DBOP_RAW	4		/* raw read			*/
+#define DBOP_KEY		1	/* read key part		*/
+#define DBOP_PREFIX		2	/* prefixed read		*/
+#define DBOP_RAW		4	/* raw read			*/
+#define DBOP_SORTED_WRITE	8	/* sorted write			*/
 
 DBOP *dbop_open(const char *, int, int, int);
 const char *dbop_get(DBOP *, const char *);
