@@ -1,21 +1,20 @@
 /*
- * Copyright (c) 2002 Tama Communications Corporation
+ * Copyright (c) 2002, 2010 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,6 +22,7 @@
 #endif
 #include <stdio.h>
 #include "split.h"
+#include "die.h"
 
 /*
  * Substring manager like perl's split.
@@ -89,9 +89,9 @@
  *	r)		part count
  */
 int
-split(char *line, int npart, SPLIT *list)
+split(const char *line, int npart, SPLIT *list)		/* virtually const */
 {
-	char *s = line;
+	char *s = (char *)line;
 	struct part *part = &list->part[0];
 	int count;
 
@@ -156,3 +156,71 @@ split_dump(SPLIT *list)
 		fprintf(stderr, "savec[%d] : |%c|\n", i, part->savec);
 	}
 }
+/*
+ * parse_xid: extract fid from ctags_xid format record.
+ *
+ *	i)	ctags_xid	ctags-xid record
+ *	o)	s_fid		file id(string) if not NULL
+ *	o)	n_fid		file id(integer) if not NULL
+ *	r)			pointer to the ctags_x part
+ */
+const char *
+parse_xid(const char *ctags_xid, char *s_fid, int *n_fid)
+{
+	const char *p;
+	int i, n;
+
+	i = n = 0;
+	for (p = ctags_xid; *p && *p >= '0' && *p <= '9'; p++) {
+		n = n * 10 + (*p - '0');
+		if (s_fid)
+			s_fid[i++] = *p;
+	}
+	if (*p++ != ' ')
+		die("illegal ctags-xid format record. '%s'", ctags_xid);
+	if (s_fid)
+		s_fid[i] = '\0';
+	if (n_fid != NULL)
+		*n_fid = n;
+	return p;
+}
+/*
+ * nextstring: seek to the next string.
+ *
+ *      i)      s       original string
+ *      r)              next string
+ *
+ *  s       v
+ * "aaaaaa\0bbbbb\0"
+ */
+const char *
+nextstring(const char *s)
+{
+	while (*s)
+		s++;
+	return s + 1;
+}
+/*
+ * nextelement: seek to the next element
+ *
+ *	i)	s	point the current element or the following blanks
+ *	r)		next element
+ *
+ *	s     s   return value
+ *	v     v   v
+ *	xxxxx     yyyyy    zzzzzz
+ *	(current) (next)
+ */
+const char *
+nextelement(const char *s)
+{
+	while (*s && !isblank(*s))
+		s++;
+	if (!*s)
+		die("nextelement: unexpected end of string(1).");
+	while (*s && isblank(*s))
+		s++;
+	if (!*s)
+		die("nextelement: unexpected end of string(2).");
+	return s;
+}
